/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// Graph.cpp

#include "stdafx.h"
#include "Graph.h"

#include <math.h>


CGraph::CGraph()
{
	m_nDataType = -1;
	m_pData1 = NULL;
	m_pData2 = NULL;
	m_pData3 = NULL;
	m_nCount = 0;
}

CGraph::~CGraph()
{
	if (m_pData1 != NULL)
		delete[] m_pData1;

	if (m_pData2 != NULL)
		delete[] m_pData2;

	if (m_pData3 != NULL)
		delete[] m_pData3;
}


void CGraph::SetXAxis(double dMin, double dMax, int nTicks)
{
	m_dMinX = dMin;
	m_dMaxX = dMax;
	m_nTicksX = nTicks;
}

void CGraph::SetYAxis(double dMin, double dMax, int nTicks)
{
	m_dMinY = dMin;
	m_dMaxY = dMax;
	m_nTicksY = nTicks;
}


void CGraph::SetLabelX(CString str)
{
	m_LabelX = str;
}

void CGraph::SetLabelY(CString str)
{
	m_LabelY = str;
}


void CGraph::CalcXAxis(double dMin, double dMax)
{
	if (dMax >= 0 && dMin >=0 && (dMax - dMin) > (dMax / 8))
	{
		// zero-based axis
		m_dMinX = 0.0;
		m_nTicksX = (int)ceil(pow(10, log10(dMax) - floor(log10(dMax))));
		m_dMaxX = pow(10, floor(log10(dMax))) * m_nTicksX;
	}
	else
	{
		double dUnits = __max(pow(10, floor(log10(fabs(dMax)))), pow(10,
			floor(log10(fabs(dMin)))));
		m_dMinX = dUnits * floor(dMin / dUnits);
		m_dMaxX = dUnits * ceil(dMax / dUnits);
		m_nTicksX = (int)(fabs(m_dMaxX - m_dMinX) / dUnits);
	}
}

void CGraph::CalcYAxis(double dMin, double dMax)
{
	if (dMax >= 0 && dMin >=0 && (dMax - dMin) > (dMax / 8))
	{
		// zero-based axis
		m_dMinY = 0.0;
		m_nTicksY = (int)ceil(pow(10, log10(dMax) - floor(log10(dMax))));
		m_dMaxY = pow(10, floor(log10(dMax))) * m_nTicksY;
	}
	else
	{
		double dUnits = __max(pow(10, floor(log10(fabs(dMax)))), pow(10,
			floor(log10(fabs(dMin)))));
		m_dMinY = dUnits * floor(dMin / dUnits);
		m_dMaxY = dUnits * ceil(dMax / dUnits);
		m_nTicksY = (int)(fabs(m_dMaxY - m_dMinY) / dUnits);
	}
}


void CGraph::SetDataPoints(double dX[], double dY[], int nCount)
{
	m_nDataType = data_points;
	m_pData1 = new double[nCount];
	m_pData2 = new double[nCount];
	memcpy(m_pData1, dX, nCount * sizeof(double));
	memcpy(m_pData2, dY, nCount * sizeof(double));
	m_nCount = nCount;
}

void CGraph::SetDataMinMax(double dMin[], double dMax[], double dTime[],
						   int nCount)
{
	m_nDataType = data_minmax;
	m_pData1 = new double[nCount];
	m_pData2 = new double[nCount];
	m_pData3 = new double[nCount];
	memcpy(m_pData1, dMin, nCount * sizeof(double));
	memcpy(m_pData2, dMax, nCount * sizeof(double));
	memcpy(m_pData3, dTime, nCount * sizeof(double));
	m_nCount = nCount;
}

void CGraph::SetData2Var(double dX[], double dY[], double dVal[], int nCount)
{
	m_nDataType = data_2var;
	m_pData1 = new double[nCount];
	m_pData2 = new double[nCount];
	m_pData3 = new double[nCount];
	memcpy(m_pData1, dX, nCount * sizeof(double));
	memcpy(m_pData2, dY, nCount * sizeof(double));
	memcpy(m_pData3, dVal, nCount * sizeof(double));
	m_nCount = nCount;
}

void CGraph::Paint(CDC *pDC)
{
	// calculate graph rectangle
	int nLeft, nRight, nTop, nBottom;
	int nWidth, nHeight;
	// this is a place for a possible error if this is used for printing
	RECT rect;
	pDC->GetWindow()->GetClientRect(&rect);
	nLeft = rect.left + 40 + 20;
	nRight = rect.right - 40;
	nTop = rect.top + 10;
	nBottom = rect.bottom - 30 - 20;
	nWidth = nRight - nLeft;
	nHeight = nBottom - nTop;

	CBrush BGBrush(RGB(255, 255, 255));
	CBrush *pOldBrush = pDC->SelectObject(&BGBrush);
	pDC->Rectangle(&rect);
	pDC->SelectObject(pOldBrush);
	
	CPen AxisPen(PS_SOLID, 1, RGB(0, 0, 0));
	CPen *pOldPen = pDC->SelectObject(&AxisPen);

	// axis
	pDC->MoveTo(nLeft, nTop);
	pDC->LineTo(nLeft, nBottom);
	pDC->LineTo(nRight, nBottom);

	// prevent any div. by zero errors
	if (m_nTicksX == 0 || m_nTicksY == 0)
		return;

	CFont NewFont;
	NewFont.CreateFont(0, 0, 899, 0, 0, FALSE, FALSE, 0, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, NULL);
	CFont *pOldFont = pDC->SelectObject(&NewFont);
	pDC->TextOut(5, nBottom - 50, m_LabelY);

	CFont NewFont2;
	NewFont2.CreateFont(0, 0, 0, 0, 0, FALSE, FALSE, 0, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, NULL);
	pDC->SelectObject(&NewFont2);
	pDC->TextOut(200, nBottom + 25, m_LabelX);

	// tick marks
	char buf[10];
	int n, z;
	for (n=0; n<=m_nTicksX; n++)
	{
		z = int(n * nWidth / m_nTicksX) + nLeft;
		pDC->MoveTo(z, nBottom + 10);
		pDC->LineTo(z, nBottom);
		_gcvt(n * (m_dMaxX - m_dMinX) / m_nTicksX + m_dMinX, 4, buf);
		pDC->TextOut(z, nBottom + 10, buf);
	}
	for (n=0; n<=m_nTicksY; n++)
	{
		z = nBottom - int(n * nHeight / m_nTicksY);
		pDC->MoveTo(nLeft - 10, z);
		pDC->LineTo(nLeft, z);
		_gcvt(n * (m_dMaxY - m_dMinY) / m_nTicksY + m_dMinY, 4, buf);
		pDC->TextOut(nLeft - 30, z + 1, buf);
	}

	// grid
	if (m_nDataType != data_2var)
	{
		CPen GridPen;
		GridPen.CreatePen(PS_SOLID, 1, RGB(168, 168, 168));
		pDC->SelectObject(&GridPen);
		for (n=1; n<=m_nTicksX; n++)
		{
			z = int(n * nWidth / m_nTicksX) + nLeft;
			pDC->MoveTo(z, nBottom);
			pDC->LineTo(z, nTop);
		}
		for (n=1; n<=m_nTicksY; n++)
		{
			z = nBottom - int(n * nHeight / m_nTicksY);
			pDC->MoveTo(nLeft, z);
			pDC->LineTo(nRight, z);
		}
	}

	// draw data
	if ( m_nCount > 0 && m_pData1 != NULL && m_pData2 != NULL &&
		m_nDataType != -1 )
	{
		CPen DataPen(PS_SOLID, 1, RGB(0, 0, 128));
		pDC->SelectObject(&DataPen);

		switch (m_nDataType)
		{
		case data_points:
			pDC->MoveTo(DataToLogicalX(m_pData1[0], nLeft, nRight),
				DataToLogicalY(m_pData2[0], nTop, nBottom));
			for (n=1; n<m_nCount; n++)
				pDC->LineTo(DataToLogicalX(m_pData1[n], nLeft, nRight),
					DataToLogicalY(m_pData2[n], nTop, nBottom));
			break;

		case data_minmax:
			for (n=0; n<m_nCount; n++)
			{
				pDC->MoveTo(DataToLogicalX(m_pData3[n], nLeft, nRight),
					DataToLogicalY(m_pData2[n], nTop, nBottom) - 1);
				pDC->LineTo(DataToLogicalX(m_pData3[n], nLeft, nRight),
					DataToLogicalY(m_pData1[n], nTop, nBottom));
			}
			break;

		case data_2var:
			PaintData2Var(pDC, nLeft, nRight, nTop, nBottom);
			// this one is too long and complex for here
			break;
		}
	}

	// grid
	if (m_nDataType == data_2var)
	{
		CPen GridPen;
		GridPen.CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
		pDC->SelectObject(&GridPen);
		for (n=1; n<=m_nTicksX; n++)
		{
			z = int(n * nWidth / m_nTicksX) + nLeft;
			pDC->MoveTo(z, nBottom);
			pDC->LineTo(z, nTop);
		}
		for (n=1; n<=m_nTicksY; n++)
		{
			z = nBottom - int(n * nHeight / m_nTicksY);
			pDC->MoveTo(nLeft, z);
			pDC->LineTo(nRight, z);
		}
	}

	pDC->SelectObject(pOldFont);
	pDC->SelectObject(pOldPen);
}


int CGraph::DataToLogicalX(double dX, int nLeft, int nRight)
{
	return (int)((dX - m_dMinX) * (nRight - nLeft) / (m_dMaxX - m_dMinX)) +
		nLeft;
}

int CGraph::DataToLogicalY(double dY, int nTop, int nBottom)
{
	return nBottom - (int)((dY - m_dMinY) * (nBottom - nTop) / (m_dMaxY -
		m_dMinY));
}

void CGraph::PaintData2Var(CDC *pDC, int nLeft, int nRight, int nTop,
						   int nBottom)
{
	int nWidth = nRight - nLeft;
	int nHeight = nBottom - nTop;
	int dim = (int)sqrt(m_nCount);
	int x, y, xp, yp;	// x & y are vertices, xp & yp are pixel coordinates
	int xs, ys, xs1, ys1;	// xl & yl are the screen coordinates of x & y
	double dVal;				// value of the pixel
	int val;					// color of the pixel
	int ws, hs;					// square width and height
	double v1, v2, v3, v4;		// data points forming square vertices
	int a1, a2, a3, a4, at;		// area of rectangle to vertices

	// we need min/max values for calculations
	double dMin = m_pData3[0];
	double dMax = m_pData3[0];
	for (int n=1; n<m_nCount; n++)
	{
		if (m_pData3[n] < dMin)
			dMin = m_pData3[n];
		if (m_pData3[n] > dMax)
			dMax = m_pData3[n];
	}
	
	// we will break the graph into squares, with the data points as vertices
	// then, we can loop through each square pixel by pixel to calculate data
	for (x=0; x<dim-1; x++)
	{
		xs = (x * nWidth / (dim - 1)) + nLeft;				// square left
		xs1 = ((x + 1) * nWidth / (dim - 1)) + nLeft;		// square right
		ws = xs1 - xs;										// square width
		for (y=0; y<dim-1; y++)
		{
			ys = (y * nHeight / (dim - 1)) + nTop;			// square top
			ys1 = ((y + 1) * nHeight / (dim - 1)) + nTop;	// square bottom
			hs = ys1 - ys;									// square height
			at = ws * hs;									// area of square

			// get vertex values
			v1 = m_pData3[GetArrayIndex(x + 1, y, dim)];
			v2 = m_pData3[GetArrayIndex(x, y, dim)];
			v3 = m_pData3[GetArrayIndex(x, y + 1, dim)];
			v4 = m_pData3[GetArrayIndex(x + 1, y + 1, dim)];

			// draw pixels
			for (xp=0; xp<ws; xp++)
			{
				for (yp=0; yp<hs; yp++)
				{
					// calc rectangle areas (coefficients are quadrants)
					a1 = (ws - xp) * yp;
					a2 = xp * yp;
					a3 = xp * (hs - yp);
					a4 = (ws - xp) * (hs - yp);

					// calculate value
					dVal = ((v1 * a3) + (v2 * a4) + (v3 * a1) + (v4 * a2)) /
						at;

					// calculate the color (from 0 - 15)
					val = int((dVal - dMin) * 15 / (dMax - dMin));
					
					// plot pixel
					pDC->SetPixel(xs + xp, ys + yp, RGB(0, 0, val * 16));
				}
			}
		}
	}
}

int CGraph::GetArrayIndex(int x, int y, int dim)
{
	// this is a very cheap shortcut to map x & y coordinates to elements in
	//   a 2-variable array, but it saves lots of processing time

	// WARNING: this code will cause erraneous graphs if the simulation order
	//   is ever changed!!!

	return (x * dim) + dim - y - 1;
}

