/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// RocketSpecsPage.cpp : implementation file
//

#include "stdafx.h"
#include "BottleRockets.h"
#include "RocketSpecsPage.h"

#include "Units.h"
#include "SimulationSettingsPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRocketSpecsPage property page

IMPLEMENT_DYNCREATE(CRocketSpecsPage, CPropertyPage)

CRocketSpecsPage::CRocketSpecsPage() : CPropertyPage(CRocketSpecsPage::IDD)
{
	//{{AFX_DATA_INIT(CRocketSpecsPage)
	m_dBottleVolume = 0.0;
	m_csDescription = _T("");
	m_dDiameter = 0.0;
	m_dDragCoefficient = 0.0;
	m_dEmptyMass = 0.0;
	m_dStability = 0.0;
	m_dParDiameter = 0.0;
	m_dParDeployTime = 0.0;
	m_nParCircular = -1;
	m_dParDragCoefficient = 0.0;
	m_bParKamikaze = FALSE;
	m_nParNumber = 0;
	m_dParSurfaceArea = 0.0;
	//}}AFX_DATA_INIT
}

CRocketSpecsPage::~CRocketSpecsPage()
{
}

void CRocketSpecsPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRocketSpecsPage)
	DDX_Control(pDX, IDC_ROCKET_PAR_SURFACEAREA_UNITS, m_ParSurfaceAreaUnits);
	DDX_Control(pDX, IDC_ROCKET_PAR_NUMBER_SPIN, m_ParNumberSpin);
	DDX_Control(pDX, IDC_ROCKET_PAR_DIAMETER_UNITS, m_ParDiameterUnits);
	DDX_Control(pDX, IDC_ROCKET_EMPTYMASS_UNITS, m_EmptyMassUnits);
	DDX_Control(pDX, IDC_ROCKET_BOTTLEVOLUME_UNITS, m_BottleVolumeUnits);
	DDX_Control(pDX, IDC_ROCKET_DIAMETER_UNITS, m_DiameterUnits);
	DDX_Text(pDX, IDC_ROCKET_BOTTLEVOLUME, m_dBottleVolume);
	DDX_Text(pDX, IDC_ROCKET_DESCRIPTION, m_csDescription);
	DDX_Text(pDX, IDC_ROCKET_DIAMETER, m_dDiameter);
	DDX_Text(pDX, IDC_ROCKET_DRAGCOEFFICIENT, m_dDragCoefficient);
	DDX_Text(pDX, IDC_ROCKET_EMPTYMASS, m_dEmptyMass);
	DDX_Text(pDX, IDC_ROCKET_STABILITY, m_dStability);
	DDX_Text(pDX, IDC_ROCKET_PAR_DIAMETER, m_dParDiameter);
	DDX_Text(pDX, IDC_ROCKET_PAR_DEPLOYTIME, m_dParDeployTime);
	DDX_Radio(pDX, IDC_ROCKET_PAR_CIRCULAR, m_nParCircular);
	DDX_Text(pDX, IDC_ROCKET_PAR_DRAGCOEFFICIENT, m_dParDragCoefficient);
	DDX_Check(pDX, IDC_ROCKET_PAR_KAMIKAZE, m_bParKamikaze);
	DDX_Text(pDX, IDC_ROCKET_PAR_NUMBER, m_nParNumber);
	DDX_Text(pDX, IDC_ROCKET_PAR_SURFACEAREA, m_dParSurfaceArea);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRocketSpecsPage, CPropertyPage)
	//{{AFX_MSG_MAP(CRocketSpecsPage)
	ON_BN_CLICKED(IDC_ROCKET_PAR_KAMIKAZE, OnRocketParKamikaze)
	ON_BN_CLICKED(IDC_ROCKET_PAR_CIRCULAR, OnRocketParCircular)
	ON_BN_CLICKED(IDC_ROCKET_PAR_IRREGULAR, OnRocketParIrregular)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRocketSpecsPage message handlers

int _PARACHUTE[] = {
	IDC_ROCKET_PAR_NUMBER,
	IDC_ROCKET_PAR_NUMBER_TEXT,
	IDC_ROCKET_PAR_NUMBER_SPIN,
	IDC_ROCKET_PAR_CIRCULAR,
	IDC_ROCKET_PAR_IRREGULAR,
	IDC_ROCKET_PAR_DRAGCOEFFICIENT,
	IDC_ROCKET_PAR_DRAGCOEFFICIENT_TEXT,
	IDC_ROCKET_PAR_DRAGCOEFFICIENT_UNITS_TEXT,
	IDC_ROCKET_PAR_DEPLOYTIME,
	IDC_ROCKET_PAR_DEPLOYTIME_TEXT,
	IDC_ROCKET_PAR_DEPLOYTIME_UNITS_TEXT };
int _CIRCULAR[] = {
	IDC_ROCKET_PAR_DIAMETER,
	IDC_ROCKET_PAR_DIAMETER_TEXT,
	IDC_ROCKET_PAR_DIAMETER_UNITS };
int _IRREGULAR[] = {
	IDC_ROCKET_PAR_SURFACEAREA,
	IDC_ROCKET_PAR_SURFACEAREA_TEXT,
	IDC_ROCKET_PAR_SURFACEAREA_UNITS };

BOOL CRocketSpecsPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	m_dDiameter = 110.0;
	FillSL(&m_DiameterUnits);
	m_DiameterUnits.SetCurSel(SL_mm);
	
	m_dBottleVolume = 2.0;
	FillVM(&m_BottleVolumeUnits);
	m_BottleVolumeUnits.SetCurSel(VM_l);

	m_dEmptyMass = 150.0;
	FillM(&m_EmptyMassUnits);
	m_EmptyMassUnits.SetCurSel(M_g);

	m_dDragCoefficient = 0.3;
	m_dStability = 1.0;

	m_bParKamikaze = TRUE;

	m_nParNumber = 1;
	m_ParNumberSpin.SetRange(1, 10);

	m_dParDiameter = 100.0;
	FillSL(&m_ParDiameterUnits);
	m_ParDiameterUnits.SetCurSel(SL_cm);

	m_dParSurfaceArea = 1.0;
	FillSA(&m_ParSurfaceAreaUnits);
	m_ParSurfaceAreaUnits.SetCurSel(SA_m2);
	
	m_nParCircular = 0;

	m_dParDragCoefficient = 1.0;
	m_dParDeployTime = 1.0;

	UpdateData(FALSE);

	UpdateDialogState();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CRocketSpecsPage::UpdateDialogState()
{
	UpdateData(TRUE);
	EnableArray(_PARACHUTE, 16, this, !m_bParKamikaze);

	EnableArray(_CIRCULAR, 3, this, !m_nParCircular && !m_bParKamikaze);
	EnableArray(_IRREGULAR, 3, this, m_nParCircular && !m_bParKamikaze);
	UpdateData(FALSE);
}

void CRocketSpecsPage::OnRocketParKamikaze() 
{
	UpdateDialogState();
}

void CRocketSpecsPage::OnRocketParCircular() 
{
	UpdateDialogState();
}

void CRocketSpecsPage::OnRocketParIrregular() 
{
	UpdateDialogState();
}
