/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// SimulatePage.cpp : implementation file
//

#include "stdafx.h"
#include "BottleRockets.h"
#include "SimulatePage.h"

//#include "LaunchAnalysisGraphWnd.h"
#include "Simulate.h"
#include "FullscreenGraph.h"
#include "Variables.h"
#include "Units.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSimulatePage property page

IMPLEMENT_DYNCREATE(CSimulatePage, CPropertyPage)

CSimulatePage::CSimulatePage() : CPropertyPage(CSimulatePage::IDD)
{
	//{{AFX_DATA_INIT(CSimulatePage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_pGraph = NULL;
}

CSimulatePage::~CSimulatePage()
{
}

void CSimulatePage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSimulatePage)
	DDX_Control(pDX, IDC_WAIT_TEXT, m_WaitText);
	DDX_Control(pDX, IDC_PRINT, m_PrintButton);
	DDX_Control(pDX, IDC_FULLSCREEN, m_FullScreenButton);
	DDX_Control(pDX, IDC_SUMMARY_TEXT, m_SummaryText);
	DDX_Control(pDX, IDC_GRAPH_LEGEND, m_GraphLegend);
	DDX_Control(pDX, IDC_GRAPH_FRAME, m_Graph);
	DDX_Control(pDX, IDC_SIMULATE_PROGRESS, m_Progress);
	DDX_Control(pDX, IDC_SIMULATE_CANCEL, m_Cancel);
	DDX_Control(pDX, IDC_SIMULATE, m_Simulate);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSimulatePage, CPropertyPage)
	//{{AFX_MSG_MAP(CSimulatePage)
	ON_BN_CLICKED(IDC_SIMULATE, OnSimulate)
	ON_BN_CLICKED(IDC_SIMULATE_CANCEL, OnSimulateCancel)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_FULLSCREEN, OnFullscreen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSimulatePage message handlers

void CSimulatePage::OnSimulate() 
{
	// we'll need this a lot, so go ahead and get a pointer to it
	CBottleRocketsApp *pApp = (CBottleRocketsApp*)AfxGetApp();
	
	UpdateData(TRUE);

	// hide old windows / graphs
	m_Graph.ShowWindow(SW_HIDE);
	m_GraphLegend.ShowWindow(SW_HIDE);
	m_SummaryText.ShowWindow(SW_HIDE);
	m_PrintButton.ShowWindow(SW_HIDE);
	m_FullScreenButton.ShowWindow(SW_HIDE);
	m_WaitText.ShowWindow(SW_SHOW);
	if (m_pGraph != NULL)
	{
		delete m_pGraph;
		m_pGraph = NULL;
	}

	double dX[_max], dY[_max], dZ[_max];
	int type;
	BOOL bGraph;	// TRUE = graph; FALSE = summary
	int len = Simulate(dX, dY, dZ, &type, &bGraph, pApp);

	m_WaitText.ShowWindow(SW_HIDE);

	if (m_bCancelled)
	{
		m_bCancelled = FALSE;
		return;
	}


	switch (type)
	{
	case data_minmax:
		// Min/Max graph used for Launch Analysis
		if (pApp->pSimulationSettingsPage->m_nLAResults == 0)
		{
			// summary
			m_SummaryText.ShowWindow(SW_SHOW);
			m_PrintButton.ShowWindow(SW_SHOW);
		}
		else
		{
			// graph
			
			// calculate these values -- CGraph will need them
			double min = dX[0];
			double max = dY[0];
			double time = dZ[0];
			for (int n=1; n<len-3; n++)	// we exclude the last piece of data
										// because it occurred a fraction of
										// a second following impact - it may
										// be erraneous
			{
				if (dX[n] < min)
					min = dX[n];
				if (dY[n] > max)
					max = dY[n];
				if (dZ[n] > time)
					time = dZ[n];
			}

			CString LabelX, LabelY;
			LabelX = "Time (sec)";
			LabelY = FV_names[pApp->pSimulationSettingsPage->m_LAPlot.GetCurSel()];
			LabelY += " (";
			int unit = pApp->pSimulationSettingsPage->m_LAPlotUnits.GetCurSel();
			switch (pApp->pSimulationSettingsPage->m_LAPlot.GetCurSel())
			{
			case FV_Accel:
				LabelY += A_units[unit];
				break;
			case FV_AirP:
				LabelY += P_units[unit];
				break;
			case FV_AirVol:
			case FV_WL:
				LabelY += VM_units[unit];
				break;
			case FV_Height:
				LabelY += LL_units[unit];
				break;
			case FV_LRF:
			case FV_Thrust:
			case FV_WR:
				LabelY += "N";
				break;
			case FV_Mass:
				LabelY += M_units[unit];
				break;
			case FV_Velocity:
				LabelY += V_units[unit];
				break;
			}
			LabelY += ")";

			// render graph
			m_Graph.ShowWindow(SW_SHOW);
			m_PrintButton.ShowWindow(SW_SHOW);
			m_FullScreenButton.ShowWindow(SW_SHOW);
			m_pGraph = new CGraph;
			m_pGraph->SetLabelX(LabelX);
			m_pGraph->SetLabelY(LabelY);
			if (time <= 20.0)
				m_pGraph->SetXAxis(0, int(time) + 1, int(time) + 1);
			else
				m_pGraph->CalcXAxis(0, time);
			m_pGraph->CalcYAxis(min, max);
			m_pGraph->SetDataMinMax(dX, dY, dZ, len);
			Invalidate();
		}
		break;

	case data_points:
		// points graph used for 1-variable optimizations
		if (pApp->pSimulationSettingsPage->m_n1VResults == 0)
		{
			// Summary
			m_SummaryText.ShowWindow(SW_SHOW);
			m_PrintButton.ShowWindow(SW_SHOW);
		}
		else
		{
			// Graph
			
			// calculate these values -- CGraph will need them
			double min = dZ[0];
			double max = dZ[0];
			for (int n=1; n<len-1; n++)	// see comment above
			{
				if (dZ[n] < min)
					min = dZ[n];
				if (dZ[n] > max)
					max = dZ[n];
			}

			CString LabelX, LabelY;

			LabelX = IV_names[pApp->pSimulationSettingsPage->m_1VIndependent.GetCurSel()];
			LabelY = DV_names[pApp->pSimulationSettingsPage->m_1VDependent.GetCurSel()];

			// render graph
			m_Graph.ShowWindow(SW_SHOW);
			m_PrintButton.ShowWindow(SW_SHOW);
			m_FullScreenButton.ShowWindow(SW_SHOW);
			m_pGraph = new CGraph;
			m_pGraph->SetLabelX(LabelX);
			m_pGraph->SetLabelY(LabelY);
			m_pGraph->SetXAxis(
				pApp->pSimulationSettingsPage->m_d1VIndependentMin,
				pApp->pSimulationSettingsPage->m_d1VIndependentMax,
				_testsperaxis - 1);
			m_pGraph->CalcYAxis(min, max);
			m_pGraph->SetDataPoints(dX, dZ, len);
			Invalidate();
		}
		break;

	case data_2var:
		// 2-variable graph used for 2-variable optimizations
		if (pApp->pSimulationSettingsPage->m_n2VResults == 0)
		{
			// Summary
			m_SummaryText.ShowWindow(SW_SHOW);
			m_PrintButton.ShowWindow(SW_SHOW);
		}
		else
		{
			CString LabelX, LabelY;
			LabelX = IV_names[pApp->pSimulationSettingsPage->m_1VIndependent.GetCurSel()];
			LabelY = DV_names[pApp->pSimulationSettingsPage->m_1VDependent.GetCurSel()];

			// Graph
			m_Graph.ShowWindow(SW_SHOW);
			m_GraphLegend.ShowWindow(SW_SHOW);
			m_PrintButton.ShowWindow(SW_SHOW);
			m_FullScreenButton.ShowWindow(SW_SHOW);
			m_pGraph = new CGraph;
			m_pGraph->SetLabelX(LabelX);
			m_pGraph->SetLabelY(LabelY);
			m_pGraph->SetXAxis(pApp->pSimulationSettingsPage->m_d2VXAxisMin,
				pApp->pSimulationSettingsPage->m_d2VXAxisMax, _testsperaxis -
				1);
			m_pGraph->SetYAxis(pApp->pSimulationSettingsPage->m_d2VYAxisMin,
				pApp->pSimulationSettingsPage->m_d2VYAxisMax, _testsperaxis -
				1);
			m_pGraph->SetData2Var(dX, dY, dZ, len);
			Invalidate();
		}
		break;
	}
}

void CSimulatePage::OnSimulateCancel() 
{
	m_bCancelled = TRUE;
}

BOOL CSimulatePage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	m_Progress.SetRange(0, 100);
	m_Progress.SetPos(0);

	m_bCancelled = FALSE;

	// hide results until a simulation is run
	m_Graph.ShowWindow(SW_HIDE);
	m_GraphLegend.ShowWindow(SW_HIDE);
	m_SummaryText.ShowWindow(SW_HIDE);
	m_PrintButton.ShowWindow(SW_HIDE);
	m_FullScreenButton.ShowWindow(SW_HIDE);
	m_WaitText.ShowWindow(SW_HIDE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSimulatePage::PumpMessages()
{
	ASSERT(m_hWnd != NULL);

	MSG msg;
	// Handle dialog messages
	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if (!IsDialogMessage(&msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
}


void CSimulatePage::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	if (m_pGraph != NULL && m_Graph.IsWindowVisible())
	{
		m_pGraph->Paint(m_Graph.GetDC());
	}

	// Do not call CPropertyPage::OnPaint() for painting messages
}

void CSimulatePage::OnFullscreen() 
{
	CFullscreenGraph wnd;
	wnd.m_pGraph = m_pGraph;
	wnd.DoModal();
}
