/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// SimulationSettingsPage.cpp : implementation file
//

#include "stdafx.h"
#include "BottleRockets.h"
#include "SimulationSettingsPage.h"

#include "Variables.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSimulationSettingsPage property page

IMPLEMENT_DYNCREATE(CSimulationSettingsPage, CPropertyPage)

CSimulationSettingsPage::CSimulationSettingsPage() : CPropertyPage(CSimulationSettingsPage::IDD)
{
	//{{AFX_DATA_INIT(CSimulationSettingsPage)
	m_nSimulationType = -1;
	m_nLAResults = -1;
	m_n2VResults = -1;
	m_n1VResults = -1;
	m_PrecisionText = _T("");
	m_d1VIndependentMax = 0.0;
	m_d1VIndependentMin = 0.0;
	m_d2VXAxisMax = 0.0;
	m_d2VXAxisMin = 0.0;
	m_d2VYAxisMax = 0.0;
	m_d2VYAxisMin = 0.0;
	//}}AFX_DATA_INIT
}

CSimulationSettingsPage::~CSimulationSettingsPage()
{
}

void CSimulationSettingsPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSimulationSettingsPage)
	DDX_Control(pDX, IDC_SIMULATION_LA_PLOT_UNITS, m_LAPlotUnits);
	DDX_Control(pDX, IDC_SIMULATION_2V_YAXIS_UNITS, m_2VYAxisUnits);
	DDX_Control(pDX, IDC_SIMULATION_2V_XAXIS_UNITS, m_2VXAxisUnits);
	DDX_Control(pDX, IDC_SIMULATION_2V_DEPENDENT_UNITS, m_2VDependentUnits);
	DDX_Control(pDX, IDC_SIMULATION_1V_INDEPENDENT_UNITS, m_1VIndependentUnits);
	DDX_Control(pDX, IDC_SIMULATION_1V_DEPENDENT_UNITS, m_1VDependentUnits);
	DDX_Control(pDX, IDC_SIMULATION_2V_YAXIS, m_2VYAxis);
	DDX_Control(pDX, IDC_SIMULATION_2V_XAXIS, m_2VXAxis);
	DDX_Control(pDX, IDC_SIMULATION_2V_DEPENDENT, m_2VDependent);
	DDX_Control(pDX, IDC_SIMULATION_1V_INDEPENDENT, m_1VIndependent);
	DDX_Control(pDX, IDC_SIMULATION_1V_DEPENDENT, m_1VDependent);
	DDX_Control(pDX, IDC_SIMULATION_LA_PLOT, m_LAPlot);
	DDX_Control(pDX, IDC_SIMULATION_PRECISION_SLIDER, m_PrecisionSlider);
	DDX_Radio(pDX, IDC_SIMULATION_LAUNCH, m_nSimulationType);
	DDX_Radio(pDX, IDC_SIMULATION_LA_SUMMARY, m_nLAResults);
	DDX_Radio(pDX, IDC_SIMULATION_2V_SUMMARY, m_n2VResults);
	DDX_Radio(pDX, IDC_SIMULATION_1V_SUMMARY, m_n1VResults);
	DDX_Text(pDX, IDC_SIMULATION_PRECISION, m_PrecisionText);
	DDX_Text(pDX, IDC_SIMULATION_1V_INDEPENDENT_MAX, m_d1VIndependentMax);
	DDX_Text(pDX, IDC_SIMULATION_1V_INDEPENDENT_MIN, m_d1VIndependentMin);
	DDX_Text(pDX, IDC_SIMULATION_2V_XAXIS_MAX, m_d2VXAxisMax);
	DDX_Text(pDX, IDC_SIMULATION_2V_XAXIS_MIN, m_d2VXAxisMin);
	DDX_Text(pDX, IDC_SIMULATION_2V_YAXIS_MAX, m_d2VYAxisMax);
	DDX_Text(pDX, IDC_SIMULATION_2V_YAXIS_MIN, m_d2VYAxisMin);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSimulationSettingsPage, CPropertyPage)
	//{{AFX_MSG_MAP(CSimulationSettingsPage)
	ON_BN_CLICKED(IDC_SIMULATION_LAUNCH, OnSimulationLaunch)
	ON_BN_CLICKED(IDC_SIMULATION_1VARIABLE, OnSimulation1Variable)
	ON_BN_CLICKED(IDC_SIMULATION_2VARIABLE, OnSimulation2Variable)
	ON_BN_CLICKED(IDC_SIMULATION_LA_SUMMARY, OnSimulationLASummary)
	ON_BN_CLICKED(IDC_SIMULATION_LA_GRAPH, OnSimulationLAGraph)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SIMULATION_PRECISION_SLIDER, OnReleasedcaptureSimulationPrecisionSlider)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_SIMULATION_PRECISION_SLIDER, OnCustomdrawSimulationPrecisionSlider)
	ON_CBN_SELCHANGE(IDC_SIMULATION_1V_DEPENDENT, OnSelchangeSimulation1vDependent)
	ON_CBN_SELCHANGE(IDC_SIMULATION_1V_INDEPENDENT, OnSelchangeSimulation1vIndependent)
	ON_CBN_SELCHANGE(IDC_SIMULATION_2V_DEPENDENT, OnSelchangeSimulation2vDependent)
	ON_CBN_SELCHANGE(IDC_SIMULATION_2V_XAXIS, OnSelchangeSimulation2vXaxis)
	ON_CBN_SELCHANGE(IDC_SIMULATION_2V_YAXIS, OnSelchangeSimulation2vYaxis)
	ON_CBN_SELCHANGE(IDC_SIMULATION_LA_PLOT, OnSelchangeSimulationLaPlot)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSimulationSettingsPage message handlers

BOOL CSimulationSettingsPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	FillComboBox(&m_LAPlot, VT_FV);
	FillComboBox(&m_1VDependent, VT_DV);
	FillComboBox(&m_1VIndependent, VT_IV);
	FillComboBox(&m_2VDependent, VT_DV);
	FillComboBox(&m_2VXAxis, VT_IV);
	FillComboBox(&m_2VYAxis, VT_IV);

	m_LAPlot.SetCurSel(FV_Height);
	m_1VDependent.SetCurSel(DV_TimeAloft);
	m_1VIndependent.SetCurSel(IV_EmptyMass);
	m_2VDependent.SetCurSel(DV_TimeAloft);
	m_2VXAxis.SetCurSel(IV_EmptyMass);
	m_2VYAxis.SetCurSel(IV_InitWL);
	UpdateData(FALSE);

	UpdateData(TRUE);
	FillUnitsComboBox(&m_LAPlotUnits, m_LAPlot.GetCurSel(), VT_FV);
	FillUnitsComboBox(&m_1VDependentUnits, m_1VDependent.GetCurSel(), VT_DV);
	FillUnitsComboBox(&m_1VIndependentUnits, m_1VIndependent.GetCurSel(),
		VT_IV);
	FillUnitsComboBox(&m_2VDependentUnits, m_2VDependent.GetCurSel(), VT_DV);
	FillUnitsComboBox(&m_2VXAxisUnits, m_2VXAxis.GetCurSel(), VT_IV);
	FillUnitsComboBox(&m_2VYAxisUnits, m_2VYAxis.GetCurSel(), VT_IV);
	UpdateData(FALSE);

	m_PrecisionSlider.SetRange(1, 8);
	m_PrecisionSlider.SetPos(3);
	m_PrecisionSlider.SetTicFreq(1);
	UpdateData(FALSE);
	UpdatePrecisionText();

	m_nSimulationType = 0;
	m_nLAResults = 1;
	m_n1VResults = 1;
	m_n2VResults = 1;
	UpdateData(FALSE);
	UpdateDialogState();

	m_d1VIndependentMin = 50.0;
	m_d1VIndependentMax = 250.0;
	m_d2VXAxisMin = 50.0;
	m_d2VXAxisMax = 250.0;
	m_d2VYAxisMin = 25.0;
	m_d2VYAxisMax = 75.0;

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// some arrays to simplify showing/hiding stuff
int _LA_MAIN[] = {
	IDC_SIMULATION_LA_RESULTSTEXT,
	IDC_SIMULATION_LA_SUMMARY,
	IDC_SIMULATION_LA_GRAPH };
int _LA_GRAPH[] = {
	IDC_SIMULATION_LA_PLOTTEXT,
	IDC_SIMULATION_LA_PLOT,
	IDC_SIMULATION_LA_PLOT_UNITS };
int _1V_MAIN[] = {
	IDC_SIMULATION_1V_RESULTSTEXT,
	IDC_SIMULATION_1V_SUMMARY,
	IDC_SIMULATION_1V_GRAPH,
	IDC_SIMULATION_1V_DEPENDENTTEXT,
	IDC_SIMULATION_1V_INDEPENDENTTEXT,
	IDC_SIMULATION_1V_RANGETEXT,
	IDC_SIMULATION_1V_TOTEXT,
	IDC_SIMULATION_1V_DEPENDENT,
	IDC_SIMULATION_1V_DEPENDENT_UNITS,
	IDC_SIMULATION_1V_INDEPENDENT,
	IDC_SIMULATION_1V_INDEPENDENT_UNITS,
	IDC_SIMULATION_1V_INDEPENDENT_MIN,
	IDC_SIMULATION_1V_INDEPENDENT_MAX };
int _2V_MAIN[] = {
	IDC_SIMULATION_2V_RESULTSTEXT,
	IDC_SIMULATION_2V_SUMMARY,
	IDC_SIMULATION_2V_GRAPH,
	IDC_SIMULATION_2V_DEPENDENTTEXT,
	IDC_SIMULATION_2V_XAXISTEXT,
	IDC_SIMULATION_2V_YAXISTEXT,
	IDC_SIMULATION_2V_RANGETEXT,
	IDC_SIMULATION_2V_TOTEXT1,
	IDC_SIMULATION_2V_TOTEXT2,
	IDC_SIMULATION_2V_DEPENDENT,
	IDC_SIMULATION_2V_DEPENDENT_UNITS,
	IDC_SIMULATION_2V_XAXIS,
	IDC_SIMULATION_2V_XAXIS_UNITS,
	IDC_SIMULATION_2V_XAXIS_MIN,
	IDC_SIMULATION_2V_XAXIS_MAX,
	IDC_SIMULATION_2V_YAXIS,
	IDC_SIMULATION_2V_YAXIS_UNITS,
	IDC_SIMULATION_2V_YAXIS_MIN,
	IDC_SIMULATION_2V_YAXIS_MAX };

void EnableArray(int array[], int nElements, CWnd *pParent,
				 BOOL bEnable /*= TRUE*/)
{
	for (int n=0; n<nElements; n++)
		pParent->GetDlgItem(array[n])->EnableWindow(bEnable);
}

void CSimulationSettingsPage::UpdateDialogState()
{
	UpdateData(TRUE);
	
	// hide everything
	EnableArray(_LA_MAIN, 3, this, FALSE);
	EnableArray(_LA_GRAPH, 3, this, FALSE);
	EnableArray(_1V_MAIN, 13, this, FALSE);
	EnableArray(_2V_MAIN, 19, this, FALSE);

	// show what is applicable
	switch (m_nSimulationType)
	{
	case 0:
		EnableArray(_LA_MAIN, 3, this, TRUE);
		if (m_nLAResults == 1)
			EnableArray(_LA_GRAPH, 3, this, TRUE);
		break;

	case 1:
		EnableArray(_1V_MAIN, 13, this, TRUE);
		break;

	case 2:
		EnableArray(_2V_MAIN, 19, this, TRUE);
		break;
	}

	UpdateData(FALSE);
}

void CSimulationSettingsPage::OnSimulationLaunch() 
{
	UpdateDialogState();
}

void CSimulationSettingsPage::OnSimulation1Variable() 
{
	UpdateDialogState();
}

void CSimulationSettingsPage::OnSimulation2Variable() 
{
	UpdateDialogState();
}

void CSimulationSettingsPage::OnSimulationLASummary() 
{
	UpdateDialogState();
}

void CSimulationSettingsPage::OnSimulationLAGraph() 
{
	UpdateDialogState();
}

void CSimulationSettingsPage::UpdatePrecisionText()
{
	UpdateData(TRUE);
	switch(m_PrecisionSlider.GetPos())
	{
	case 1:
		m_PrecisionText = "1/10th second";
		break;

	case 2:
		m_PrecisionText = "1/100th second";
		break;

	case 3:
		m_PrecisionText = "1 millisecond";
		break;

	case 4:
		m_PrecisionText = "1/10th millisecond";
		break;

	case 5:
		m_PrecisionText = "1/100th millisecond";
		break;

	case 6:
		m_PrecisionText = "1 nanosecond";
		break;

	case 7:
		m_PrecisionText = "1/10th nanosecond";
		break;

	case 8:
		m_PrecisionText = "1/100th nanosecond";
		break;
	}
	UpdateData(FALSE);
}

void CSimulationSettingsPage::OnReleasedcaptureSimulationPrecisionSlider(NMHDR* pNMHDR, LRESULT* pResult)
{
	UpdatePrecisionText();
	*pResult = 0;
}

void CSimulationSettingsPage::OnCustomdrawSimulationPrecisionSlider(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UpdatePrecisionText();	
	*pResult = 0;
}

void CSimulationSettingsPage::OnSelchangeSimulation1vDependent() 
{
	UpdateData(TRUE);
	FillUnitsComboBox(&m_1VDependentUnits, m_1VDependent.GetCurSel(), VT_DV);
	UpdateData(FALSE);
}

void CSimulationSettingsPage::OnSelchangeSimulation1vIndependent() 
{
	UpdateData(TRUE);
	FillUnitsComboBox(&m_1VIndependentUnits, m_1VIndependent.GetCurSel(), VT_IV);
	UpdateData(FALSE);
}

void CSimulationSettingsPage::OnSelchangeSimulation2vDependent() 
{
	UpdateData(TRUE);
	FillUnitsComboBox(&m_2VDependentUnits, m_2VDependent.GetCurSel(), VT_DV);
	UpdateData(FALSE);
}

void CSimulationSettingsPage::OnSelchangeSimulation2vXaxis() 
{
	UpdateData(TRUE);
	FillUnitsComboBox(&m_2VXAxisUnits, m_2VXAxis.GetCurSel(), VT_IV);
	UpdateData(FALSE);
}

void CSimulationSettingsPage::OnSelchangeSimulation2vYaxis() 
{
	UpdateData(TRUE);
	FillUnitsComboBox(&m_2VYAxisUnits, m_2VYAxis.GetCurSel(), VT_IV);
	UpdateData(FALSE);
}

void CSimulationSettingsPage::OnSelchangeSimulationLaPlot() 
{
	UpdateData(TRUE);
	FillUnitsComboBox(&m_LAPlotUnits, m_LAPlot.GetCurSel(), VT_FV);
	UpdateData(FALSE);
}

