/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// Units.cpp

#include "stdafx.h"
#include "Units.h"


// small length units
const char *SL_units[]	= {
	"cm",
	"in",
	"mm" };
const double SL_conv[]	= {
	10.0,			// cm
	0.39370079,		// in
	1.0 };			// mm

double ConvSL(double value, int inputunits, int outputunits)
{ return value * SL_conv[outputunits] / SL_conv[inputunits]; }

void FillSL(CComboBox *pComboBox)
{
	for (int n=0; n<SL_count; n++)
		pComboBox->AddString(SL_units[n]);
}


// large length units
const char *LL_units[]	= {
	"ft",
	"m",
	"yds" };
const double LL_conv[]	= {
	3.28083990,		// ft
	1.0,			// m
	1.09361330 };	// yds

double ConvLL(double value, int inputunits, int outputunits)
{ return value * LL_conv[outputunits] / LL_conv[inputunits]; }

void FillLL(CComboBox *pComboBox)
{
	for (int n=0; n<LL_count; n++)
		pComboBox->AddString(LL_units[n]);
}


// velocity units
const char *V_units[]	= {
	"ft/sec",
	"ft/min",
	"m/sec",
	"MPH",
	"km/h" };
const double V_conv[]	= {
	3.28083990,		// ft/sec
	196.85039370,	// ft/min
	1.0,			// m/sec
	2.23693629,		// MPH
	3.6 };			// km/h

double ConvV(double value, int inputunits, int outputunits)
{ return value * V_conv[outputunits] / V_conv[inputunits]; }

void FillV(CComboBox *pComboBox)
{
	for (int n=0; n<V_count; n++)
		pComboBox->AddString(V_units[n]);
}


// acceleration units
const char *A_units[]	= {
	"ft/sec^2",
	"ft/min^2",
	"G's",
	"m/sec^2",
	"mi/hr^2",
	"km/h^2" };
const double A_conv[]	= {
	3.28083990,		// ft/sec^2
	11811.02364,	// ft/min^2
	0.10197838,		// G's
	1.0,			// m/sec^2
	8052.970664,	// mi/hr^2
	12960.0 };		// km/hr^2

double ConvA(double value, int inputunits, int outputunits)
{ return value * A_conv[outputunits] / A_conv[inputunits]; }

void FillA(CComboBox *pComboBox)
{
	for (int n=0; n<A_count; n++)
		pComboBox->AddString(A_units[n]);
}


// temperature units
const char *T_units[]	= {
	"deg C",
	"deg F",
	"K" };
const double AbsZero	= -273.15;

double ConvT(double value, int inputunits, int outputunits)
{
	double result = value;
	
	switch(inputunits)
	{
	case T_degf:
		result = (5.0/9.0) * (value - 32.0);
		break;

	case T_kelvin:
		result = value - AbsZero;
		break;
	}

	switch(outputunits)
	{
	case T_degf:
		result = ((9.0/5.0) * value) + 32.0;
		break;
	
	case T_kelvin:
		result += AbsZero;
		break;
	}

	return result;
}

void FillT(CComboBox *pComboBox)
{
	for (int n=0; n<T_count; n++)
		pComboBox->AddString(T_units[n]);
}


// density units
const char *D_units[]	= {
	"g/cm^3",
	"kg/L",
	"kg/m^3",
	"lbs/ft^3" };
const double D_conv[]	= {
	0.001,			// g/cm^3
	1000.0,			// kg/L
	1.0,			// kg/m^3
	0.06243865 };	// lbs/ft^3

double ConvD(double value, int inputunits, int outputunits)
{ return value * D_conv[outputunits] / D_conv[inputunits]; }

void FillD(CComboBox *pComboBox)
{
	for (int n=0; n<D_count; n++)
		pComboBox->AddString(D_units[n]);
}


// molar mass units
const char *MM_units[]	= {
	"g/mol",
	"kg/mol",
	"lbs/mol" };
const double MM_conv[]	= {
	1000.0,			// g/mol
	1.0,			// kg/mol
	2.205 };		// lbs/mol

double ConvMM(double value, int inputunits, int outputunits)
{ return value * MM_conv[outputunits] / MM_conv[inputunits]; }

void FillMM(CComboBox *pComboBox)
{
	for (int n=0; n<MM_count; n++)
		pComboBox->AddString(MM_units[n]);
}


// volume units
const char *VM_units[]	= {
	"gal",
	"L",
	"m^3",
	"mL",
	"qt" };
const double VM_conv[]	= {
	0.000264,		// gal
	1000.0,			// L
	1.0,			// m^3
	1000000.0,		// mL
	0.001056 };		// qt

double ConvVM(double value, int inputunits, int outputunits)
{ return value * VM_conv[outputunits] / VM_conv[inputunits]; }

void FillVM(CComboBox *pComboBox)
{
	for (int n=0; n<VM_count; n++)
		pComboBox->AddString(VM_units[n]);
}


// mass units
const char *M_units[]	= {
	"g",
	"kg",
	"lbs",
	"oz" };
const double M_conv[]	= {
	1000.0,			// g
	1.0,			// kg
	2.205,			// lbs
	35.274 };		// oz

double ConvM(double value, int inputunits, int outputunits)
{ return value * M_conv[outputunits] / M_conv[inputunits]; }

void FillM(CComboBox *pComboBox)
{
	for (int n=0; n<M_count; n++)
		pComboBox->AddString(M_units[n]);
}


// pressure
const char *P_units[]	= {
	"atm",
	"in Hg",
	"kPa",
	"Pa",
	"PSI",
	"torr" };
const double P_conv[]	= {
	1.0,			// atm
	29.92125984,	// in Hg
	101.325,		// kPa
	101325.0,		// Pa
	14.70,			// PSI
	760.0 };		// torr

double ConvP(double value, int inputunits, int outputunits)
{ return value * P_conv[outputunits] / P_conv[inputunits]; }

void FillP(CComboBox *pComboBox)
{
	for (int n=0; n<P_count; n++)
		pComboBox->AddString(P_units[n]);
}


// surface area
const char *SA_units[]	= {
	"cm^2",
	"in^2",
	"ft^2",
	"m^2",
	"yds^2" };
const double SA_conv[]	= {
	10000.0,		// cm^2
	1550.0031,		// in^2
	10.76391042,	// ft^2
	1.0,			// m^2
	1.19599005 };	// yds^2

double ConvSA(double value, int inputunits, int outputunits)
{ return value * P_conv[outputunits] / P_conv[inputunits]; }

void FillSA(CComboBox *pComboBox)
{
	for (int n=0; n<SA_count; n++)
		pComboBox->AddString(SA_units[n]);
}

