/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// Variables.cpp

#include "stdafx.h"
#include "Variables.h"
#include "Units.h"

const char *FV_names[]	= {	
	"Acceleration",
	"Air Pressure",
	"Air Volume",
	"Height",
	"Launch Rod Force",
	"Mass",
	"Thrust",
	"Velocity",
	"Water Level",
	"Wind Resistance" };
const int FV_count		= 10;

// dependent variables
const char *DV_names[]		= {
	"Apogee Time",
	"Burnout Time",
	"Impact Velocity",
	"Max. Acceleration",
	"Max. Height",
	"Max. Velocity",
	"Time Aloft" };
const int DV_count			= 7;

// independent variables
const char *IV_names[]		= {
	"Air Temperature",
	"Atm. Pressure",
	"Atm. Temperature",
	"Bottle Volume",
	"Diameter",
	"Drag Coefficient",
	"Empty Mass",
	"Gravity",
	"Initial Air Pressure",
	"Initial Height",
	"Initial Velocity",
	"Initial Water Level",
	"Launch Rod Diameter",
	"Launch Rod Height",
	"Par. Deploy Time",
	"Par. Diameter",
	"Par. Drag Coefficient",
	"Par., Number of",
	"Par. Surface Area",
	"Water Density" };
const int IV_count			= 20;

void FillComboBox(CComboBox *pComboBox, int nType)
{
	int n;
	switch (nType)
	{
	case VT_FV:
		for (n=0; n<FV_count; n++)
			pComboBox->AddString(FV_names[n]);
		break;
	
	case VT_DV:
		for (n=0; n<DV_count; n++)
			pComboBox->AddString(DV_names[n]);
		break;

	case VT_IV:
		for (n=0; n<IV_count; n++)
			pComboBox->AddString(IV_names[n]);
		break;
	}
}

void FillUnitsComboBox(CComboBox *pComboBox, int nVariable, int nType)
{
	pComboBox->ResetContent();

	switch (nType)
	{
	case VT_FV:
		switch (nVariable)
		{
		case FV_Height:
			// large length units
			FillLL(pComboBox);
			pComboBox->SetCurSel(LL_m);
			break;

		case FV_Velocity:
			// velocity units
			FillV(pComboBox);
			pComboBox->SetCurSel(V_mps);
			break;

		case FV_Accel:
			// acceleration units
			FillA(pComboBox);
			pComboBox->SetCurSel(A_mps2);
			break;

		case FV_AirVol:
		case FV_WL:
			// volume units
			FillVM(pComboBox);
			pComboBox->AddString("%");
			pComboBox->SetCurSel(VM_count);
			break;

		case FV_Mass:
			// mass units
			FillM(pComboBox);
			pComboBox->SetCurSel(M_g);
			break;

		case FV_AirP:
			// pressure units
			FillP(pComboBox);
			pComboBox->SetCurSel(P_psi);
			break;

		case FV_LRF:
		case FV_Thrust:
		case FV_WR:
			// force units
			pComboBox->AddString("N");
			pComboBox->SetCurSel(0);
			break;
		}
		break;
	
	case VT_DV:
		switch (nVariable)
		{
		case DV_MaxAccel:
			// acceleration units
			FillA(pComboBox);
			pComboBox->SetCurSel(A_mps2);
			break;

		case DV_MaxVelocity:
		case DV_ImpactVelocity:
			// velocity units
			FillV(pComboBox);
			pComboBox->SetCurSel(V_mps);
			break;

		case DV_MaxHeight:
			// height units
			FillLL(pComboBox);
			pComboBox->SetCurSel(LL_m);
			break;

		case DV_ApogeeTime:
		case DV_BurnoutTime:
		case DV_TimeAloft:
			// time units
			pComboBox->AddString("sec");
			pComboBox->SetCurSel(0);
			break;
		}
		break;

	case VT_IV:
		switch (nVariable)
		{
		case IV_Diameter:
		case IV_LRDiameter:
		case IV_LRHeight:
			// small length units
			FillSL(pComboBox);
			pComboBox->SetCurSel(SL_mm);
			break;

		case IV_ParDiameter:
			// special case of small length units
			FillSL(pComboBox);
			pComboBox->SetCurSel(SL_cm);
			break;

		case IV_InitHeight:
			// large length units
			FillLL(pComboBox);
			pComboBox->SetCurSel(LL_m);
			break;

		case IV_InitVelocity:
			// velocity units
			FillV(pComboBox);
			pComboBox->SetCurSel(V_mps);
			break;

		case IV_AirT:
		case IV_AtmT:
			// temperature units
			FillT(pComboBox);
			pComboBox->SetCurSel(T_degc);
			break;

		case IV_BottleVol:
			// volume units
			FillVM(pComboBox);
			pComboBox->SetCurSel(VM_l);
			break;

		case IV_InitWL:
			// also volume units, but an exception
			FillVM(pComboBox);
			pComboBox->AddString("%");
			pComboBox->SetCurSel(VM_count);
			break;

		case IV_EmptyMass:
			// mass units
			FillM(pComboBox);
			pComboBox->SetCurSel(M_g);
			break;

		case IV_AtmP:
		case IV_InitAirP:
			// pressure units
			FillP(pComboBox);
			if (nVariable == IV_InitAirP)
				pComboBox->SetCurSel(P_psi);
			else
				pComboBox->SetCurSel(P_atm);
			break;

		case IV_Cd:
		case IV_ParCd:
			// drag coefficient
			pComboBox->AddString("Cd");
			pComboBox->SetCurSel(0);
			break;

		case IV_ParDeployTime:
			// time units
			pComboBox->AddString("sec");
			pComboBox->SetCurSel(0);
			break;

		case IV_ParNumber:
			// count
			pComboBox->AddString("#");
			pComboBox->SetCurSel(0);
			break;

		case IV_ParSA:
			// surface area units
			FillSA(pComboBox);
			pComboBox->SetCurSel(SA_m2);
			break;

		case IV_WaterD:
			// density units
			FillD(pComboBox);
			pComboBox->SetCurSel(D_kgpm3);
			break;

		case IV_Gravity:
			// acceleration units
			FillA(pComboBox);
			pComboBox->SetCurSel(A_mps2);
			break;
		}
	}
}

