/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// forces.h
// defines and calculates forces related to rocket

#if !defined(__FORCES_H)
#define __FORCES_H

#include "state.h"

class CForce
{
public:
	CForce(CState *pNewState);
	virtual ~CForce();

	virtual double Simulate(double dt) = 0;

protected:
	CState *pState;
};

class CArchimedesForce : public CForce
{
public:
	CArchimedesForce(CState *pNewState);
	virtual ~CArchimedesForce();

	virtual double Simulate(double dt);
};

class CLaunchRodForce : public CForce
{
public:
	CLaunchRodForce(CState *pNewState);
	virtual ~CLaunchRodForce();

	virtual double Simulate(double dt);
};

class CAtmosphericPressureForce : public CForce
{
public:
	CAtmosphericPressureForce(CState *pNewState);
	virtual ~CAtmosphericPressureForce();

	virtual double Simulate(double dt);
};

class CWindResistanceForce : public CForce
{
public:
	CWindResistanceForce(CState *pNewState);
	virtual ~CWindResistanceForce();

	virtual double Simulate(double dt);
};

class CGravitationForce : public CForce
{
public:
	CGravitationForce(CState *pNewState);
	virtual ~CGravitationForce();

	virtual double Simulate(double dt);
};

class CWaterForce : public CForce
{
public:
	CWaterForce(CState *pNewState);
	virtual ~CWaterForce();

	virtual double Simulate(double dt);
};

class CParachuteForce : public CForce
{
public:
	CParachuteForce(CState *pNewState);
	virtual ~CParachuteForce();

	virtual double Simulate(double dt);

protected:
	double m_dDeployCounter;	// in sec
};


#endif // __FORCES_H
