/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// state.cpp
// defines rocket and environment state

#include "stdafx.h"
#include <math.h>
#include "state.h"

CState::CState()
{
	Gravity = -9.806; // m/sec^2
	WaterDensity = 1000.0; // kg/m^3
	AirMolarMass = 0.029; // kg/mol
}

CState::~CState()
{
}

void CState::SetInitialAirVolume()
{
	InitialAirVolume = CalcAirVolume();
}

double CState::CalcAirDensity(double Pressure, double Temperature)
{
	// Pressure in atm, Temperature in deg C, Result in kg/m^3
	// (formula derived from ideal gas law)	
	return ((Pressure * AirMolarMass) / (IdealGasConstant * (Temperature -
		AbsoluteZero))) * 1000;
}

double CState::CalcAtmosphericAirDensity()
{
	// Result in kg/m^3
	return CalcAirDensity(AtmosphericPressure, AtmosphericTemperature);
}

double CState::CalcRodPerpendicularSA()
{
	// Result in m^2
	return PI * pow((LaunchRodDiameter / 2000), 2);
}

double CState::CalcRocketPerpendicularSA()
{
	// Result in m^2
	return PI * pow((BottleDiameter / 2000), 2);
}

double CState::CalcRodInBottle()
{
	// Result in mm
	double rod = LaunchRodHeight - (Height * 1000);
	if ( rod > 0.0 )
		return rod;
	else
		return 0;
}

double CState::CalcBottleVolumeMinusRod()
{
	// Result in L
	// (note: m^2 * mm yields dm^3)
	return BottleVolume - (CalcRodPerpendicularSA() * LaunchRodHeight);
}

double CState::CalcWaterVolume()
{
	// Result in L
	return BottleVolume * WaterLevel;
}

double CState::CalcAirVolume()
{
	// Result in L
	return CalcBottleVolumeMinusRod() - CalcWaterVolume();
}

double CState::CalcAirPressure()
{
	// Result in Pa
	if (WaterLevel != 0.0)
		return InitialAirPressure * InitialAirVolume / CalcAirVolume();
	else
		return AtmosphericPressure * Atm2Pascals;
}

double CState::CalcTotalMass()
{
	// Result in kg
	double sum = EmptyMass;
	if ( WaterLevel > 0.0 )
	{
		sum += CalcWaterVolume() * WaterDensity / 1000;
		sum += CalcAirDensity(InitialAirPressure / Atm2Pascals,
			AirTemperature) * InitialAirVolume / 1000;
	}
	return sum;
}

double CState::CalcWaterHeight()
{
	// Result in m
	return WaterLevel * BottleVolume * 1000 / (PI * pow(BottleDiameter / 2,
		2));
}

