/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Columbus High School Bottle Rocket Simulator                           //
//  Copyright (C) 1999-2000 Leo C. Singleton IV                            //
//    Internet: http://www.leosolutions.com/                               //
//    E-Mail: leo@leosolutions.com                                         //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// state.h
// defines rocket and environment state

#if !defined(__STATE_H)
#define __STATE_H

// constants
const double PI = 3.1415926535897932;
const double IdealGasConstant = 0.0820568; // atm*L/mol*K
const double AbsoluteZero = -273.15; // deg C

// conversion factors
const double Atm2Pascals = 101325;

class CState
{
public:
	CState();
	virtual ~CState();

	// constants
	double Gravity;					// in m/sec^2
	double WaterDensity;			// in kg/m^3
	double AirMolarMass;			// in kg/mol
	
	// environment
	double AtmosphericPressure;		// in atm
	double AtmosphericTemperature;	// in deg C
	double Updraft;					// in m/sec

	// launch pad
	double LaunchRodDiameter;		// in mm
	double LaunchRodHeight;			// in mm

	// rocket
	double Height;					// in m
	double Acceleration;			// in m/sec^2
	double Velocity;				// in m/sec
	double WaterLevel;				// range 0.0 - 1.0

	double BottleDiameter;			// in mm
	double BottleVolume;			// in L

	double DragCoefficient;			// Cd (0.0 - 1.0)

	double InitialAirPressure;		// in Pa
	double InitialAirVolume;		// in L (set by SetInitialAirVolume)
	double InitialWaterLevel;		// range 0.0 - 1.0

	double AirTemperature;			// in deg C

	double EmptyMass;				// in kg

	// parachute
	BOOL ParachuteKamikaze;			// disables parachute forces
	int ParachuteCount;				// number of parachutes
	double ParachuteSurfaceArea;	// in m^2
	double ParachuteDragCoefficient;// Cd
	double ParachuteDeployTime;		// in sec

	// functions
	void SetInitialAirVolume();	// (only call before simulation begins)

	double CalcAirDensity(double Pressure, double Temperature);
		// Pressure in atm, Temperature in deg C, Result in kg/m^3
	double CalcAtmosphericAirDensity();	// Result in kg/m^3

	double CalcRodPerpendicularSA();	// Result in m^2
	double CalcRocketPerpendicularSA();	// Result in m^2
	
	double CalcRodInBottle();			// Result in mm
	double CalcBottleVolumeMinusRod();	// Result in L
	double CalcWaterVolume();			// Result in L
	double CalcAirVolume();				// Result in L
	double CalcAirPressure();			// Result in Pa

	double CalcTotalMass();				// Result in kg

	double CalcWaterHeight();			// Result in m
};

#endif // __STATE_H
